import {useComputedState, useEffect} from './index';
import {isBrowser, off, on} from './misc/util';
import {ComputedRef} from "vue";

const patchHistoryMethod = (method) => {
    const history = window.history;
    const original = history[method];

    history[method] = function (state) {
        const result = original.apply(this, arguments);
        const event = new Event(method.toLowerCase());

        (event as any).state = state;

        window.dispatchEvent(event);

        return result;
    };
};

if (isBrowser) {
    patchHistoryMethod('pushState');
    patchHistoryMethod('replaceState');
}

export interface LocationSensorState {
    trigger: string;
    state?: any;
    length?: number;
    hash?: string;
    host?: string;
    hostname?: string;
    href?: string;
    origin?: string;
    pathname?: string;
    port?: string;
    protocol?: string;
    search?: string;
}

const useLocationServer = (): LocationSensorState => ({
    trigger: 'load',
    length: 1,
});

const buildState = (trigger: string) => {
    const {state, length} = window.history;

    const {hash, host, hostname, href, origin, pathname, port, protocol, search} = window.location;

    return {
        trigger,
        state,
        length,
        hash,
        host,
        hostname,
        href,
        origin,
        pathname,
        port,
        protocol,
        search,
    };
};

const useLocationBrowser = (): ComputedRef<LocationSensorState> => {
    const [state, setState] = useComputedState(buildState('load'));

    useEffect(() => {
        const onPopstate = () => setState(buildState('popstate'));
        const onPushstate = () => setState(buildState('pushstate'));
        const onReplacestate = () => setState(buildState('replacestate'));

        on(window, 'popstate', onPopstate);
        on(window, 'pushstate', onPushstate);
        on(window, 'replacestate', onReplacestate);

        return () => {
            off(window, 'popstate', onPopstate);
            off(window, 'pushstate', onPushstate);
            off(window, 'replacestate', onReplacestate);
        };
    }, []);

    return state;
};

const hasEventConstructor = typeof Event === 'function';

export default isBrowser && hasEventConstructor ? useLocationBrowser : useLocationServer;
